/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import { AlertInterface, AlertLevels, IdentifiableComponentInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import {
    ContentLoader,
    EmphasizedSegment,
    Heading
} from "@wso2is/react-components";
import { RequestErrorInterface } from "apps/console/src/features/core/hooks/use-request";
import { AxiosError } from "axios";
import React, {
    FunctionComponent,
    ReactElement,
    useEffect,
    useState
} from "react";
import { useTranslation } from "react-i18next";
import { useDispatch } from "react-redux";
import { Dispatch } from "redux";
import { Divider, Grid } from "semantic-ui-react";
import { SubscribedAPIResources } from ".";
import { history } from "../../../../../features/core";
import { useAPIResources } from "../../../api-resources/api";
import { APIResourcesConstants } from "../../../api-resources/constants";
import { useSubscribedAPIResources } from "../../api";


/**
 * Application roles component.
 * 
 * @param props - Props related to application roles component.
 */
export const APIAuthorization: FunctionComponent<IdentifiableComponentInterface> = (
    props: IdentifiableComponentInterface
): ReactElement => {

    const {
        ["data-componentid"]: componentId
    } = props;

    const { t } = useTranslation();
    const dispatch: Dispatch = useDispatch();

    const [ isLoading, setIsLoading ] = useState<boolean>(false);
    const [ isShownError, setIsShownError ] = useState<boolean>(false);

    const path: string[] = history.location.pathname.split("/");
    const appId: string = path[path.length - 1].split("#")[0];

    const {
        data: allAPIResourcesListData,
        isLoading: isAllAPIResourcesListLoading,
        error: allAPIResourcesFetchRequestError
    } = useAPIResources();

    const {
        data: subscribedAPIResourcesListData,
        isLoading: isSubscribedAPIResourcesListLoading,
        error: subscribedAPIResourcesFetchRequestError
    } = useSubscribedAPIResources(appId);

    /**
     * The following useEffect is used to handle if any error occurs while fetching API resources
     */
    useEffect(() => {
        if (allAPIResourcesFetchRequestError || subscribedAPIResourcesFetchRequestError) {
            if(!isShownError) {
                setIsShownError(true);

                const error: AxiosError<RequestErrorInterface> = allAPIResourcesFetchRequestError ? 
                    allAPIResourcesFetchRequestError : subscribedAPIResourcesFetchRequestError;
                
                switch (error.response?.data?.code) {
                    case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".unauthorizedError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".unauthorizedError.message")
                        }));
    
                        break;
    
                    default:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".genericError.description"),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.apiResource.notifications.getAPIResources" +
                                ".genericError.message")
                        }));
                }
            }
        }
    }, [ allAPIResourcesFetchRequestError, subscribedAPIResourcesFetchRequestError ]);

    /**
     * The following useEffect is used to handle loading state of when API resources are being fetched.
     */
    useEffect(() => {
        setIsLoading(isAllAPIResourcesListLoading || isSubscribedAPIResourcesListLoading);
    }, [ isAllAPIResourcesListLoading, isSubscribedAPIResourcesListLoading ]);

    /**
     * Handles the hide of the policy selection./
     */
    const isHidePolicySelection = (): boolean => {
        return ( subscribedAPIResourcesFetchRequestError !== null || allAPIResourcesFetchRequestError !== null 
            || allAPIResourcesListData.apiResources.length == 0 || subscribedAPIResourcesListData.length == 0 );
    };

    return (
        <EmphasizedSegment padded="very" data-componentid={ componentId }>
            <Grid>
                <Grid.Row>
                    <Grid.Column className="heading-wrapper" computer={ 10 }>
                        <Heading as="h4">
                            { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" + 
                                ".apiSubscriptions.heading") }
                        </Heading>
                        <Heading subHeading ellipsis as="h6" >
                            { t("extensions:develop.applications.edit.sections.apiAuthorization.sections" + 
                                ".apiSubscriptions.subHeading") }
                        </Heading>
                    </Grid.Column >
                </Grid.Row>
            </Grid>
            <Divider hidden />
            <Grid>
                <Grid.Row>
                    <Grid.Column>
                        {
                            isLoading 
                                ? <ContentLoader inline="centered" active/>
                                : (<SubscribedAPIResources 
                                    allAPIResourcesListData={ allAPIResourcesListData?.apiResources } 
                                    allAPIResourcesFetchRequestError={ allAPIResourcesFetchRequestError } 
                                    subscribedAPIResourcesListData={ subscribedAPIResourcesListData } 
                                    subscribedAPIResourcesFetchRequestError={ subscribedAPIResourcesFetchRequestError } 
                                    isShownPlaceholder={ isHidePolicySelection() } 
                                />)
                        }
                    </Grid.Column>
                </Grid.Row>
            </Grid>
        </EmphasizedSegment>
    );
};

/**
 * Default props for application roles tab component.
 */
APIAuthorization.defaultProps = {
    "data-componentid": "api-authorization-tab"
};
